'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

var _mapValues = require('lodash/mapValues');

var _mapValues2 = _interopRequireDefault(_mapValues);

var _isomorphicFetch = require('isomorphic-fetch');

var _isomorphicFetch2 = _interopRequireDefault(_isomorphicFetch);

var _httpErrors = require('http-errors');

var _httpErrors2 = _interopRequireDefault(_httpErrors);

var _route = require('./route');

var _route2 = _interopRequireDefault(_route);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

_isomorphicFetch2.default.Promise = Promise;

/**
 * @class BaseSdk
 * @param opts
 */

var BaseSdk = (function () {
  function BaseSdk(opts) {
    _classCallCheck(this, BaseSdk);

    if (typeof opts === 'string') {
      opts = { url: opts };
    }

    var url = opts.url;

    if (!url) {
      throw new Error('Service base url must be specified');
    }

    this._baseUrl = (url.indexOf('http') !== 0 ? 'http://' + url : url).replace(/\/+$/, '');
    this._opts = opts;
  }

  /**
   * Process http request.
   * @param {string} method The http method (get, post, put, delete, head, etc).
   * @param {string} url The service endpoint url.
   * @param {Object=} request The request object.
   * @returns {Promise<{response: Object, body: Object|[]}>}
   * @protected
   */

  _createClass(BaseSdk, [{
    key: 'fetch',
    value: function fetch(method, url) {
      var request = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

      var headers = Object.assign({
        'Accept': 'application/json',
        'Content-Type': 'application/json'
      }, request.headers || {});

      method = method.toLowerCase();

      var params = {
        method: method,
        headers: headers,
        credentials: 'same-origin'
      };

      if ((method === 'post' || method === 'put') && request.body) {
        params.body = JSON.stringify(request.body);
      }

      return (0, _isomorphicFetch2.default)(url, params).catch(function () {
        throw _httpErrors2.default.InternalServerError('Server is unavailable');
      }).then(this.assertResponse);
    }

    /**
     * Asser response
     * @param {Object=} response The response object.
     * @returns {Promise.<{response: Object, body: Object|[]}>}
     * @protected
     */

  }, {
    key: 'assertResponse',
    value: function assertResponse(response) {
      var status = response.status;

      if (status < 400) {
        return status === 204 ? null : response.json();
      }

      return response.json().then(function (error) {
        throw (0, _httpErrors2.default)(status, error);
      });
    }

    /**
     * Resolves path to full service absolute url.
     * @param {string=} path The service path
     * @returns {string}
     */

  }, {
    key: 'url',
    value: function url(path) {
      return this._baseUrl.concat('/').concat((path || '').replace(/^\/+/, ''));
    }
  }]);

  return BaseSdk;
})();

/**
 * SDK class factory method. Creates a class where methods are generated by specified
 * route config object.
 * @param {object} sdkConfig The object where values are {@link Route} instances.
 * @returns {function}
 */

function sdkFactory(sdkConfig) {
  /**
   * Generated SDK class.
   * @constructor
   */

  var Sdk = (function (_BaseSdk) {
    _inherits(Sdk, _BaseSdk);

    function Sdk() {
      _classCallCheck(this, Sdk);

      return _possibleConstructorReturn(this, Object.getPrototypeOf(Sdk).apply(this, arguments));
    }

    return Sdk;
  })(BaseSdk);

  Sdk.errors = _httpErrors2.default;

  var routes = (0, _mapValues2.default)(sdkConfig, routeFactory);
  Object.assign(Sdk.prototype, (0, _mapValues2.default)(routes, routeMethod));

  return Sdk;
}

/**
 * Creates a route instance
 * @param routeConfig
 * @param name
 * @returns {Route}
 */
function routeFactory(routeConfig, name) {
  if (typeof routeConfig === 'string') {
    routeConfig = { path: routeConfig };
  }

  routeConfig.name = name;

  return new _route2.default(routeConfig);
}

/**
 * Create request method for SDK library that corresponds to a specified route.
 * @param {Route} route
 * @returns {function(request:Object)}
 */
function routeMethod(route) {
  /**
   * Generated request method that corresponds to a specified route.
   * @param {{params:Object=, query:Object=, data:Object=, headers:Object=}=} request
   * @this {BaseSdk}
   * @returns {Promise.<Object|[]>}
   */
  return function (request) {
    request = request || {};
    var path = route.pathFormat(request);

    return this.fetch(route.method, this.url(path), request);
  };
}

exports.default = sdkFactory;