import qs from 'qs';
import pick from 'lodash/pick';
import omit from 'lodash/omit';
import union from 'lodash/union';
import findKey from 'lodash/findKey';
import isObject from 'lodash/isPlainObject';
import isUndefined from 'lodash/isUndefined';

const pathParamRe = /:([\d\w\-_]+)/g;
const httpMethodsRe = {
  get: /get|fetch|find|query/i,
  post: /post|create|send/,
  put: /put|update/,
  delete: /delete|remove|destroy/
};

const paginationParams = ['page', 'perPage'];

export default class Route {
  constructor(opts) {
    this._opts = opts || {};
    this.path = this._opts.path;
    this.name = this._opts.name || 'action';

    this.method = this._suggestMethod();
    this.queryParams = this._queryParams();
  }

  pathFormat(request) {
    let path = null;
    let queryString = null;
    let query = pick(request, this.queryParams);

    if (isObject(request)) {
      path = this.path.replace(pathParamRe, (match, group) => request[group] || match);
    }

    if (isObject(query)) {
      query = omit(query, isUndefined);
      queryString = qs.stringify(query);
    }

    return queryString ? `${path || ''}?${queryString}` : path;
  }

  _suggestMethod() {
    const method = this._opts.method;

    return method || findKey(httpMethodsRe, methodRe => methodRe.test(this.name)) || 'get';
  }

  _queryParams() {
    let queryParams = [];

    if (this._opts.pagination) {
      queryParams = union(queryParams, paginationParams);
    }

    if (this._opts.queryParams) {
      queryParams = union(queryParams, this._opts.queryParams);
    }

    return queryParams;
  }
}
